@@ -6,7 +6,6 @@ use warnings;
 use JSON;
 use Encode;
 use Data::Dumper;
-
 use HTTP::Request::Common qw(POST GET);
 use LWP::UserAgent;
 use LWP::Protocol::https;
@@ -38,7 +37,7 @@ sub connect {
 	my $self = shift;
 	my ($token) = @_;
 
-	my $res = $ua->request(POST 'https://slack.com/api/rtm.start', [ token => $token ]);
+	my $res = $ua->request(POST 'https://slack.com/api/rtm.connect', [ token => $token ]);
 	my $content;
 	eval {
 		$content = JSON::from_json($res->content);
@@ -46,7 +45,7 @@ sub connect {
 	if ($@) {
 		die 'connect response fail:'.Dumper $res->content;
 	}
-	die 'connect response fail: '.$res->content unless ($content->{ok});
+	die 'connect response fail 01: '.$res->content unless ($content->{ok});
 
 	$self->{info} = Slack::RTM::Bot::Information->new(%{$content});
 	$res = $ua->request(POST 'https://slack.com/api/conversations.list ', [ token => $token ]);
@@ -54,9 +53,9 @@ sub connect {
 		$content = JSON::decode_json($res->content);
 	};
 	if ($@) {
-		die 'connect response fail:'.Dumper $res->content;
+		die 'connect response fail 02:'.Dumper $res->content;
 	}
-	die 'connect response fail: '.$res->content unless ($content->{ok});
+	die 'connect response fail 03: '.$res->content unless ($content->{ok});
 
 	for my $im (@{$content->{channels}}) {
 		$self->{info}->{channels}->{$im->{id}} = { %$im, name => '@'.$im->{name} };
@@ -155,23 +154,31 @@ sub _refetch_conversation_name {
 
 sub _refetch_conversations {
 	my $self = shift;
-	my $res;
-	eval {
-		my $conversations = {};
-		my $cursor = "";
-		do {
-			$res = $ua->request(GET "https://slack.com/api/conversations.list?types=public_channel,private_channel&token=$self->{token}&cursor=$cursor&limit=1000");
-			my $args = JSON::from_json($res->content);
-			for my $conversation (@{$args->{channels}}) {
-				$conversations->{$conversation->{id}} = $conversation;
+	my $cursor = "";
+	do {
+		my $res = $ua->request(POST 'https://slack.com/api/conversations.list', [ token => $self->{token}, types => "public_channel,private_channel,im", cursor => $cursor ]);
+		my $content;
+		eval {
+			$content = JSON::decode_json($res->content);
+		};
+		if ($@) {
+			die 'connect response fail:' . Dumper $res->content;
+		}
+		die 'connect response fail: ' . $res->content unless ($content->{ok});
+
+		for my $channel (@{$content->{channels}}) {
+			if ($channel->{is_im}) {
+				my $user_id = $channel->{user};
+				my $name = $self->{info}->_find_user_name($user_id);
+				$name ||= $self->_refetch_user_name($user_id) or warn "There are no users of such id: $user_id";
+				$self->{info}->{channels}->{$channel->{id}} = { %$channel, name => '@'.$name };
+				next;
 			}
-			$cursor = $args->{response_metadata}->{next_cursor};
-		} until ($cursor eq "");
-		$self->{info}->{channels} = $conversations;
-       };
-       if ($@) {
-	       die '_refetch_conversations response fail:'.Dumper $res->content;
-       }
+			$self->{info}->{channels}->{$channel->{id}} = $channel;
+		}
+
+		$cursor = $content->{response_metadata}->{next_cursor};
+	} until ($cursor eq "");
 }
 
 sub find_user_name {
@@ -206,10 +213,12 @@ sub _refetch_users {
 		do {
 			$res = $ua->request(GET "https://slack.com/api/users.list?token=$self->{token}&cursor=$cursor");
 			my $args = JSON::from_json($res->content);
-			for my $user (@{$args->{users}}) {
+			for my $user (@{$args->{members}}) {
 				$users->{$user->{id}} = $user;
 			}
-			$cursor = $args->{response_metadata}->{next_cursor};
+			if (defined($args->{response_metadata}->{next_cursor})) {
+				$cursor = $args->{response_metadata}->{next_cursor};
+			}
 		} until ($cursor eq "");
 		$self->{info}->{users} = $users;
        };
